@@ -1,15 +1,12 @@
 use std::{env, process::exit};
 
 use matrix_sdk::{
-    room::Room,
-    ruma::events::{room::member::MemberEventContent, StrippedStateEvent},
-    Client, ClientConfig, SyncSettings,
+    config::SyncSettings, room::Room, ruma::events::room::member::StrippedRoomMemberEvent, Client,
 };
 use tokio::time::{sleep, Duration};
-use url::Url;
 
 async fn on_stripped_state_member(
-    room_member: StrippedStateEvent<MemberEventContent>,
+    room_member: StrippedRoomMemberEvent,
     client: Client,
     room: Room,
 ) {
@@ -43,14 +40,26 @@ async fn login_and_sync(
     homeserver_url: String,
     username: &str,
     password: &str,
-) -> Result<(), matrix_sdk::Error> {
-    let mut home = dirs::home_dir().expect("no home directory found");
-    home.push("autojoin_bot");
+) -> anyhow::Result<()> {
+    #[allow(unused_mut)]
+    let mut client_builder = Client::builder().homeserver_url(homeserver_url);
 
-    let client_config = ClientConfig::new().store_path(home);
+    #[cfg(feature = "sled")]
+    {
+        // The location to save files to
+        let mut home = dirs::home_dir().expect("no home directory found");
+        home.push("autojoin_bot");
+        let state_store = matrix_sdk_sled::StateStore::open_with_path(home)?;
+        client_builder = client_builder.state_store(Box::new(state_store));
+    }
 
-    let homeserver_url = Url::parse(&homeserver_url).expect("Couldn't parse the homeserver URL");
-    let client = Client::new_with_config(homeserver_url, client_config).unwrap();
+    #[cfg(feature = "indexeddb")]
+    {
+        let state_store = matrix_sdk_indexeddb::StateStore::open();
+        client_builder = client_builder.state_store(Box::new(state_store));
+    }
+
+    let client = client_builder.build().await?;
 
     client.login(username, password, None, Some("autojoin bot")).await?;
 
@@ -64,7 +73,7 @@ async fn login_and_sync(
 }
 
 #[tokio::main]
-async fn main() -> Result<(), matrix_sdk::Error> {
+async fn main() -> anyhow::Result<()> {
     tracing_subscriber::fmt::init();
 
     let (homeserver_url, username, password) =
