@@ -4,27 +4,30 @@ use std::{
     sync::atomic::{AtomicBool, Ordering},
 };
 
-use matrix_sdk::{ruma::UserId, Client, LoopCtrl, SyncSettings};
+use matrix_sdk::{config::SyncSettings, ruma::OwnedUserId, Client, LoopCtrl};
 use url::Url;
 
-async fn bootstrap(client: Client, user_id: UserId, password: String) {
+async fn bootstrap(client: Client, user_id: OwnedUserId, password: String) {
     println!("Bootstrapping a new cross signing identity, press enter to continue.");
 
     let mut input = String::new();
 
     io::stdin().read_line(&mut input).expect("error: unable to read user input");
 
-    #[cfg(feature = "encryption")]
-    if let Err(e) = client.bootstrap_cross_signing(None).await {
-        use matrix_sdk::ruma::{api::client::r0::uiaa, assign};
+    if let Err(e) = client.encryption().bootstrap_cross_signing(None).await {
+        use matrix_sdk::ruma::{api::client::uiaa, assign};
 
         if let Some(response) = e.uiaa_response() {
             let auth_data = uiaa::AuthData::Password(assign!(
-                uiaa::Password::new(uiaa::UserIdentifier::MatrixId(user_id.as_str()), &password),
+                uiaa::Password::new(
+                    uiaa::UserIdentifier::UserIdOrLocalpart(user_id.as_str()),
+                    &password,
+                ),
                 { session: response.session.as_deref() }
             ));
 
             client
+                .encryption()
                 .bootstrap_cross_signing(Some(auth_data))
                 .await
                 .expect("Couldn't bootstrap cross signing")
@@ -32,9 +35,6 @@ async fn bootstrap(client: Client, user_id: UserId, password: String) {
             panic!("Error during cross-signing bootstrap {:#?}", e);
         }
     }
-
-    #[cfg(not(feature = "encryption"))]
-    panic!("Cross signing requires the encryption feature to be enabled");
 }
 
 async fn login(
@@ -43,7 +43,7 @@ async fn login(
     password: &str,
 ) -> Result<(), matrix_sdk::Error> {
     let homeserver_url = Url::parse(&homeserver_url).expect("Couldn't parse the homeserver URL");
-    let client = Client::new(homeserver_url).unwrap();
+    let client = Client::new(homeserver_url).await.unwrap();
 
     let response = client.login(username, password, None, Some("rust-sdk")).await?;
 
@@ -57,15 +57,10 @@ async fn login(
             let asked = asked_ref;
             let client = &client_ref;
             let user_id = &user_id;
-            let password = &password;
 
             // Wait for sync to be done then ask the user to bootstrap.
             if !asked.load(Ordering::SeqCst) {
-                tokio::spawn(bootstrap(
-                    (*client).clone(),
-                    (*user_id).clone(),
-                    password.to_string(),
-                ));
+                tokio::spawn(bootstrap((*client).clone(), (*user_id).clone(), password.to_owned()));
             }
 
             asked.store(true, Ordering::SeqCst);
