@@ -9,15 +9,17 @@ use std::{
 
 use matrix_sdk::{
     self,
+    config::SyncSettings,
+    encryption::verification::{SasVerification, Verification},
     ruma::{
         events::{
-            room::message::MessageType, AnySyncMessageEvent, AnySyncRoomEvent, AnyToDeviceEvent,
+            room::message::MessageType, AnySyncMessageLikeEvent, AnySyncRoomEvent, AnyToDeviceEvent,
         },
         UserId,
     },
-    verification::{SasVerification, Verification},
-    Client, LoopCtrl, SyncSettings,
+    Client, LoopCtrl,
 };
+use ruma::events::SyncMessageLikeEvent;
 use url::Url;
 
 async fn wait_for_confirmation(client: Client, sas: SasVerification) {
@@ -53,11 +55,11 @@ fn print_result(sas: &SasVerification) {
 async fn print_devices(user_id: &UserId, client: &Client) {
     println!("Devices of user {}", user_id);
 
-    for device in client.get_user_devices(user_id).await.unwrap().devices() {
+    for device in client.encryption().get_user_devices(user_id).await.unwrap().devices() {
         println!(
             "   {:<10} {:<30} {:<}",
             device.device_id(),
-            device.display_name().as_deref().unwrap_or_default(),
+            device.display_name().unwrap_or("-"),
             device.verified()
         );
     }
@@ -69,7 +71,7 @@ async fn login(
     password: &str,
 ) -> Result<(), matrix_sdk::Error> {
     let homeserver_url = Url::parse(&homeserver_url).expect("Couldn't parse the homeserver URL");
-    let client = Client::new(homeserver_url).unwrap();
+    let client = Client::new(homeserver_url).await.unwrap();
 
     client.login(username, password, None, Some("rust-sdk")).await?;
 
@@ -85,8 +87,10 @@ async fn login(
             for event in response.to_device.events.iter().filter_map(|e| e.deserialize().ok()) {
                 match event {
                     AnyToDeviceEvent::KeyVerificationStart(e) => {
-                        if let Some(Verification::SasV1(sas)) =
-                            client.get_verification(&e.sender, &e.content.transaction_id).await
+                        if let Some(Verification::SasV1(sas)) = client
+                            .encryption()
+                            .get_verification(&e.sender, e.content.transaction_id.as_str())
+                            .await
                         {
                             println!(
                                 "Starting verification with {} {}",
@@ -99,16 +103,20 @@ async fn login(
                     }
 
                     AnyToDeviceEvent::KeyVerificationKey(e) => {
-                        if let Some(Verification::SasV1(sas)) =
-                            client.get_verification(&e.sender, &e.content.transaction_id).await
+                        if let Some(Verification::SasV1(sas)) = client
+                            .encryption()
+                            .get_verification(&e.sender, e.content.transaction_id.as_str())
+                            .await
                         {
                             tokio::spawn(wait_for_confirmation((*client).clone(), sas));
                         }
                     }
 
                     AnyToDeviceEvent::KeyVerificationMac(e) => {
-                        if let Some(Verification::SasV1(sas)) =
-                            client.get_verification(&e.sender, &e.content.transaction_id).await
+                        if let Some(Verification::SasV1(sas)) = client
+                            .encryption()
+                            .get_verification(&e.sender, e.content.transaction_id.as_str())
+                            .await
                         {
                             if sas.is_done() {
                                 print_result(&sas);
@@ -126,12 +134,15 @@ async fn login(
                     for event in
                         room_info.timeline.events.iter().filter_map(|e| e.event.deserialize().ok())
                     {
-                        if let AnySyncRoomEvent::Message(event) = event {
+                        if let AnySyncRoomEvent::MessageLike(event) = event {
                             match event {
-                                AnySyncMessageEvent::RoomMessage(m) => {
+                                AnySyncMessageLikeEvent::RoomMessage(
+                                    SyncMessageLikeEvent::Original(m),
+                                ) => {
                                     if let MessageType::VerificationRequest(_) = &m.content.msgtype
                                     {
                                         let request = client
+                                            .encryption()
                                             .get_verification_request(&m.sender, &m.event_id)
                                             .await
                                             .expect("Request object wasn't created");
@@ -142,8 +153,11 @@ async fn login(
                                             .expect("Can't accept verification request");
                                     }
                                 }
-                                AnySyncMessageEvent::KeyVerificationKey(e) => {
+                                AnySyncMessageLikeEvent::KeyVerificationKey(
+                                    SyncMessageLikeEvent::Original(e),
+                                ) => {
                                     if let Some(Verification::SasV1(sas)) = client
+                                        .encryption()
                                         .get_verification(
                                             &e.sender,
                                             e.content.relates_to.event_id.as_str(),
@@ -153,8 +167,11 @@ async fn login(
                                         tokio::spawn(wait_for_confirmation((*client).clone(), sas));
                                     }
                                 }
-                                AnySyncMessageEvent::KeyVerificationMac(e) => {
+                                AnySyncMessageLikeEvent::KeyVerificationMac(
+                                    SyncMessageLikeEvent::Original(e),
+                                ) => {
                                     if let Some(Verification::SasV1(sas)) = client
+                                        .encryption()
                                         .get_verification(
                                             &e.sender,
                                             e.content.relates_to.event_id.as_str(),
