@@ -9,25 +9,22 @@ use std::{
 
 use matrix_sdk::{
     self,
+    attachment::AttachmentConfig,
+    config::SyncSettings,
     room::Room,
-    ruma::events::{
-        room::message::{MessageEventContent, MessageType, TextMessageEventContent},
-        SyncMessageEvent,
+    ruma::events::room::message::{
+        MessageType, OriginalSyncRoomMessageEvent, RoomMessageEventContent, TextMessageEventContent,
     },
-    Client, SyncSettings,
+    Client,
 };
 use tokio::sync::Mutex;
 use url::Url;
 
-async fn on_room_message(
-    event: SyncMessageEvent<MessageEventContent>,
-    room: Room,
-    image: Arc<Mutex<File>>,
-) {
+async fn on_room_message(event: OriginalSyncRoomMessageEvent, room: Room, image: Arc<Mutex<File>>) {
     if let Room::Joined(room) = room {
-        let msg_body = if let SyncMessageEvent {
+        let msg_body = if let OriginalSyncRoomMessageEvent {
             content:
-                MessageEventContent {
+                RoomMessageEventContent {
                     msgtype: MessageType::Text(TextMessageEventContent { body: msg_body, .. }),
                     ..
                 },
@@ -43,7 +40,9 @@ async fn on_room_message(
             println!("sending image");
             let mut image = image.lock().await;
 
-            room.send_attachment("cat", &mime::IMAGE_JPEG, &mut *image, None).await.unwrap();
+            room.send_attachment("cat", &mime::IMAGE_JPEG, &mut *image, AttachmentConfig::new())
+                .await
+                .unwrap();
 
             image.seek(SeekFrom::Start(0)).unwrap();
 
@@ -59,7 +58,7 @@ async fn login_and_sync(
     image: File,
 ) -> Result<(), matrix_sdk::Error> {
     let homeserver_url = Url::parse(&homeserver_url).expect("Couldn't parse the homeserver URL");
-    let client = Client::new(homeserver_url).unwrap();
+    let client = Client::new(homeserver_url).await.unwrap();
 
     client.login(&username, &password, None, Some("command bot")).await?;
 
