@@ -2,20 +2,20 @@ use std::{env, process::exit};
 
 use matrix_sdk::{
     self,
+    config::SyncSettings,
     room::Room,
-    ruma::events::{
-        room::message::{MessageEventContent, MessageType, TextMessageEventContent},
-        SyncMessageEvent,
+    ruma::events::room::message::{
+        MessageType, OriginalSyncRoomMessageEvent, RoomMessageEventContent, TextMessageEventContent,
     },
-    Client, SyncSettings,
+    Client,
 };
 use url::Url;
 
-async fn on_room_message(event: SyncMessageEvent<MessageEventContent>, room: Room) {
+async fn on_room_message(event: OriginalSyncRoomMessageEvent, room: Room) {
     if let Room::Joined(room) = room {
-        if let SyncMessageEvent {
+        if let OriginalSyncRoomMessageEvent {
             content:
-                MessageEventContent {
+                RoomMessageEventContent {
                     msgtype: MessageType::Text(TextMessageEventContent { body: msg_body, .. }),
                     ..
                 },
@@ -36,7 +36,7 @@ async fn login(
     password: &str,
 ) -> Result<(), matrix_sdk::Error> {
     let homeserver_url = Url::parse(&homeserver_url).expect("Couldn't parse the homeserver URL");
-    let client = Client::new(homeserver_url).unwrap();
+    let client = Client::new(homeserver_url).await.unwrap();
 
     client.register_event_handler(on_room_message).await;
 
