@@ -17,7 +17,9 @@
 use std::io::Error as IoError;
 
 use http::StatusCode;
-#[cfg(feature = "encryption")]
+#[cfg(feature = "qrcode")]
+use matrix_sdk_base::crypto::ScanError;
+#[cfg(feature = "e2e-encryption")]
 use matrix_sdk_base::crypto::{
     CryptoStoreError, DecryptorError, KeyExportError, MegolmError, OlmError,
 };
@@ -25,24 +27,35 @@ use matrix_sdk_base::{Error as SdkBaseError, StoreError};
 use reqwest::Error as ReqwestError;
 use ruma::{
     api::{
-        client::{
-            r0::uiaa::{UiaaInfo, UiaaResponse as UiaaError},
-            Error as RumaClientApiError,
-        },
-        error::{FromHttpResponseError, IntoHttpError, MatrixError as RumaApiError, ServerError},
+        client::uiaa::{UiaaInfo, UiaaResponse as UiaaError},
+        error::{FromHttpResponseError, IntoHttpError, ServerError},
     },
-    identifiers::Error as IdentifierError,
+    events::tag::InvalidUserTagName,
+    IdParseError,
 };
 use serde_json::Error as JsonError;
 use thiserror::Error;
 use url::ParseError as UrlParseError;
 
-/// Result type of the rust-sdk.
-pub type Result<T> = std::result::Result<T, Error>;
+/// Result type of the matrix-sdk.
+pub type Result<T, E = Error> = std::result::Result<T, E>;
 
 /// Result type of a pure HTTP request.
 pub type HttpResult<T> = std::result::Result<T, HttpError>;
 
+/// An error response from a Matrix API call, using a client API specific
+/// representation if the endpoint is from that.
+#[derive(Error, Debug)]
+pub enum RumaApiError {
+    /// A client API response error.
+    #[error(transparent)]
+    ClientApi(ruma::api::client::Error),
+
+    /// Another API response error.
+    #[error(transparent)]
+    Other(ruma::api::error::MatrixError),
+}
+
 /// An HTTP error, representing either a connection error or an error while
 /// converting the raw HTTP response into a Matrix response.
 #[derive(Error, Debug)]
@@ -56,22 +69,13 @@ pub enum HttpError {
     #[error("the queried endpoint requires authentication but was called before logging in")]
     AuthenticationRequired,
 
-    /// Client tried to force authentication but did not provide an access
-    /// token.
-    #[error("tried to force authentication but no access token was provided")]
-    ForcedAuthenticationWithoutAccessToken,
-
     /// Queried endpoint is not meant for clients.
     #[error("the queried endpoint is not meant for clients")]
     NotClientRequest,
 
     /// An error converting between ruma_*_api types and Hyper types.
     #[error(transparent)]
-    Api(#[from] FromHttpResponseError<RumaApiError>),
-
-    /// An error converting between ruma_client_api types and Hyper types.
-    #[error(transparent)]
-    ClientApi(#[from] FromHttpResponseError<RumaClientApiError>),
+    Api(FromHttpResponseError<RumaApiError>),
 
     /// An error converting between ruma_client_api types and Hyper types.
     #[error(transparent)]
@@ -101,6 +105,7 @@ pub enum HttpError {
 
 /// Internal representation of errors.
 #[derive(Error, Debug)]
+#[non_exhaustive]
 pub enum Error {
     /// Error doing an HTTP request.
     #[error(transparent)]
@@ -111,6 +116,17 @@ pub enum Error {
     #[error("the queried endpoint requires authentication but was called before logging in")]
     AuthenticationRequired,
 
+    /// Attempting to restore a session after the olm-machine has already been
+    /// set up fails
+    #[cfg(feature = "e2e-encryption")]
+    #[error("The olm machine has already been initialized")]
+    BadCryptoStoreState,
+
+    /// Attempting to access the olm-machine but it is not yet available.
+    #[cfg(feature = "e2e-encryption")]
+    #[error("The olm machine isn't yet available")]
+    NoOlmMachine,
+
     /// An error de/serializing type for the `StateStore`
     #[error(transparent)]
     SerdeJson(#[from] JsonError),
@@ -120,26 +136,22 @@ pub enum Error {
     Io(#[from] IoError),
 
     /// An error occurred in the crypto store.
-    #[cfg(feature = "encryption")]
-    #[cfg_attr(feature = "docs", doc(cfg(encryption)))]
+    #[cfg(feature = "e2e-encryption")]
     #[error(transparent)]
     CryptoStoreError(#[from] CryptoStoreError),
 
     /// An error occurred during a E2EE operation.
-    #[cfg(feature = "encryption")]
-    #[cfg_attr(feature = "docs", doc(cfg(encryption)))]
+    #[cfg(feature = "e2e-encryption")]
     #[error(transparent)]
     OlmError(#[from] OlmError),
 
     /// An error occurred during a E2EE group operation.
-    #[cfg(feature = "encryption")]
-    #[cfg_attr(feature = "docs", doc(cfg(encryption)))]
+    #[cfg(feature = "e2e-encryption")]
     #[error(transparent)]
     MegolmError(#[from] MegolmError),
 
     /// An error occurred during decryption.
-    #[cfg(feature = "encryption")]
-    #[cfg_attr(feature = "docs", doc(cfg(encryption)))]
+    #[cfg(feature = "e2e-encryption")]
     #[error(transparent)]
     DecryptorError(#[from] DecryptorError),
 
@@ -149,16 +161,35 @@ pub enum Error {
 
     /// An error encountered when trying to parse an identifier.
     #[error(transparent)]
-    Identifier(#[from] IdentifierError),
+    Identifier(#[from] IdParseError),
 
     /// An error encountered when trying to parse a url.
     #[error(transparent)]
     Url(#[from] UrlParseError),
+
+    /// An error while scanning a QR code.
+    #[cfg(feature = "qrcode")]
+    #[error(transparent)]
+    QrCodeScanError(#[from] ScanError),
+
+    /// An error encountered when trying to parse a user tag name.
+    #[error(transparent)]
+    UserTagName(#[from] InvalidUserTagName),
+
+    /// An error while processing images.
+    #[cfg(feature = "image-proc")]
+    #[error(transparent)]
+    ImageError(#[from] ImageError),
+
+    /// An other error was raised
+    /// this might happen because encryption was enabled on the base-crate
+    /// but not here and that raised.
+    #[error("unknown error: {0}")]
+    UnknownError(Box<dyn std::error::Error + Send + Sync>),
 }
 
 /// Error for the room key importing functionality.
-#[cfg(feature = "encryption")]
-#[cfg_attr(feature = "docs", doc(cfg(encryption)))]
+#[cfg(feature = "e2e-encryption")]
 #[derive(Error, Debug)]
 // This is allowed because key importing isn't enabled under wasm.
 #[allow(dead_code)]
@@ -198,7 +229,7 @@ impl HttpError {
     /// This method is an convenience method to get to the info the server
     /// returned on the first, failed request.
     pub fn uiaa_response(&self) -> Option<&UiaaInfo> {
-        if let HttpError::UiaaError(FromHttpResponseError::Http(ServerError::Known(
+        if let HttpError::UiaaError(FromHttpResponseError::Server(ServerError::Known(
             UiaaError::AuthResponse(i),
         ))) = self
         {
@@ -222,9 +253,9 @@ impl Error {
     /// This method is an convenience method to get to the info the server
     /// returned on the first, failed request.
     pub fn uiaa_response(&self) -> Option<&UiaaInfo> {
-        if let Error::Http(HttpError::UiaaError(FromHttpResponseError::Http(ServerError::Known(
-            UiaaError::AuthResponse(i),
-        )))) = self
+        if let Error::Http(HttpError::UiaaError(FromHttpResponseError::Server(
+            ServerError::Known(UiaaError::AuthResponse(i)),
+        ))) = self
         {
             Some(i)
         } else {
@@ -233,6 +264,18 @@ impl Error {
     }
 }
 
+impl From<FromHttpResponseError<ruma::api::client::Error>> for HttpError {
+    fn from(err: FromHttpResponseError<ruma::api::client::Error>) -> Self {
+        Self::Api(err.map(|e| e.map(RumaApiError::ClientApi)))
+    }
+}
+
+impl From<FromHttpResponseError<ruma::api::error::MatrixError>> for HttpError {
+    fn from(err: FromHttpResponseError<ruma::api::error::MatrixError>) -> Self {
+        Self::Api(err.map(|e| e.map(RumaApiError::Other)))
+    }
+}
+
 impl From<SdkBaseError> for Error {
     fn from(e: SdkBaseError) -> Self {
         match e {
@@ -240,12 +283,23 @@ impl From<SdkBaseError> for Error {
             SdkBaseError::StateStore(e) => Self::StateStore(e),
             SdkBaseError::SerdeJson(e) => Self::SerdeJson(e),
             SdkBaseError::IoError(e) => Self::Io(e),
-            #[cfg(feature = "encryption")]
+            #[cfg(feature = "e2e-encryption")]
             SdkBaseError::CryptoStore(e) => Self::CryptoStoreError(e),
-            #[cfg(feature = "encryption")]
+            #[cfg(feature = "e2e-encryption")]
+            SdkBaseError::BadCryptoStoreState => Self::BadCryptoStoreState,
+            #[cfg(feature = "e2e-encryption")]
             SdkBaseError::OlmError(e) => Self::OlmError(e),
-            #[cfg(feature = "encryption")]
+            #[cfg(feature = "e2e-encryption")]
             SdkBaseError::MegolmError(e) => Self::MegolmError(e),
+            #[cfg(feature = "eyre")]
+            _ => Self::UnknownError(eyre::eyre!(e).into()),
+            #[cfg(all(not(feature = "eyre"), feature = "anyhow"))]
+            _ => Self::UnknownError(anyhow::anyhow!(e).into()),
+            #[cfg(all(not(feature = "eyre"), not(feature = "anyhow")))]
+            _ => {
+                let e: Box<dyn std::error::Error + Sync + Send> = format!("{:?}", e).into();
+                Self::UnknownError(e)
+            }
         }
     }
 }
@@ -255,3 +309,20 @@ impl From<ReqwestError> for Error {
         Error::Http(HttpError::Reqwest(e))
     }
 }
+
+/// All possible errors that can happen during image processing.
+#[cfg(feature = "image-proc")]
+#[derive(Error, Debug)]
+pub enum ImageError {
+    /// Error processing the image data.
+    #[error(transparent)]
+    Proc(#[from] image::ImageError),
+
+    /// The image format is not supported.
+    #[error("the image format is not supported")]
+    FormatNotSupported,
+
+    /// The thumbnail size is bigger than the original image.
+    #[error("the thumbnail size is bigger than the original image size")]
+    ThumbnailBiggerThanOriginal,
+}
