@@ -24,13 +24,15 @@
 //!
 //! The stored closure takes care of deserializing the event which the
 //! `EventHandlerData` contains as a (borrowed) [`serde_json::value::RawValue`],
-//! extracing the context arguments from other fields of `EventHandlerData` and
+//! extracting the context arguments from other fields of `EventHandlerData` and
 //! calling / `.await`ing the event handler if the previous steps succeeded.
 //! It also logs any errors from the above chain of function calls.
 //!
 //! For more details, see the [`EventHandler`] trait.
 
-use std::{borrow::Cow, future::Future, ops::Deref};
+#[cfg(any(feature = "anyhow", feature = "eyre"))]
+use std::any::TypeId;
+use std::{borrow::Cow, fmt, future::Future, ops::Deref};
 
 use matrix_sdk_base::deserialized_responses::{EncryptionInfo, SyncRoomEvent};
 use ruma::{events::AnySyncStateEvent, serde::Raw};
@@ -45,14 +47,36 @@ pub enum EventKind {
     GlobalAccountData,
     RoomAccountData,
     EphemeralRoomData,
-    Message { redacted: bool },
-    State { redacted: bool },
+    MessageLike,
+    OriginalMessageLike,
+    RedactedMessageLike,
+    State,
+    OriginalState,
+    RedactedState,
     StrippedState,
     InitialState,
     ToDevice,
     Presence,
 }
 
+impl EventKind {
+    fn message_like_redacted(redacted: bool) -> Self {
+        if redacted {
+            Self::RedactedMessageLike
+        } else {
+            Self::OriginalMessageLike
+        }
+    }
+
+    fn state_redacted(redacted: bool) -> Self {
+        if redacted {
+            Self::RedactedState
+        } else {
+            Self::OriginalState
+        }
+    }
+}
+
 /// A statically-known event kind/type that can be retrieved from an event sync.
 pub trait SyncEvent {
     #[doc(hidden)]
@@ -67,9 +91,10 @@ pub trait SyncEvent {
 /// * They must have at least one argument, which is the event itself, a type
 ///   that implements [`SyncEvent`]. Any additional arguments need to implement
 ///   the [`EventHandlerContext`] trait.
-/// * Their return type has to be one of: `()`, `Result<(), impl
-///   std::error::Error>` or `anyhow::Result<()>` (requires the `anyhow` Cargo
-///   feature to be enabled)
+/// * Their return type has to be one of: `()`, `Result<(), impl Display + Debug
+///   + 'static>` (if you are using `anyhow::Result` or `eyre::Result` you can
+///   additionally enable the `anyhow` / `eyre` feature to get the verbose
+///   `Debug` output printed on error)
 ///
 /// ### How it works
 ///
@@ -176,6 +201,25 @@ impl EventHandlerContext for Option<EncryptionInfo> {
     }
 }
 
+/// A custom value registered with
+/// [`.register_event_handler_context`][Client::register_event_handler_context].
+#[derive(Debug)]
+pub struct Ctx<T>(pub T);
+
+impl<T: Clone + Send + Sync + 'static> EventHandlerContext for Ctx<T> {
+    fn from_data(data: &EventHandlerData<'_>) -> Option<Self> {
+        data.client.event_handler_context::<T>().map(Ctx)
+    }
+}
+
+impl<T> Deref for Ctx<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 /// Return types supported for event handlers implement this trait.
 ///
 /// It is not meant to be implemented outside of matrix-sdk.
@@ -188,19 +232,21 @@ impl EventHandlerResult for () {
     fn print_error(&self, _event_type: &str) {}
 }
 
-impl<E: std::error::Error> EventHandlerResult for Result<(), E> {
+impl<E: fmt::Debug + fmt::Display + 'static> EventHandlerResult for Result<(), E> {
     fn print_error(&self, event_type: &str) {
-        if let Err(e) = self {
-            tracing::error!("Event handler for `{}` failed: {}", event_type, e);
-        }
-    }
-}
-
-#[cfg(feature = "anyhow")]
-impl EventHandlerResult for anyhow::Result<()> {
-    fn print_error(&self, event_type: &str) {
-        if let Err(e) = self {
-            tracing::error!("Event handler for `{}` failed: {:?}", event_type, e);
+        match self {
+            #[cfg(feature = "anyhow")]
+            Err(e) if TypeId::of::<E>() == TypeId::of::<anyhow::Error>() => {
+                tracing::error!("Event handler for `{}` failed: {:?}", event_type, e);
+            }
+            #[cfg(feature = "eyre")]
+            Err(e) if TypeId::of::<E>() == TypeId::of::<eyre::Report>() => {
+                tracing::error!("Event handler for `{}` failed: {:?}", event_type, e);
+            }
+            Err(e) => {
+                tracing::error!("Event handler for `{}` failed: {}", event_type, e);
+            }
+            Ok(_) => {}
         }
     }
 }
@@ -228,7 +274,7 @@ impl Client {
             room,
             events,
             |ev| (ev, None),
-            |raw| Ok((kind, raw.deserialize_as::<ExtractType>()?.event_type)),
+            |raw| Ok((kind, raw.deserialize_as::<ExtractType<'_>>()?.event_type)),
         )
         .await
     }
@@ -245,6 +291,10 @@ impl Client {
             unsigned: Option<UnsignedDetails>,
         }
 
+        // Event handlers for possibly-redacted state events
+        self.handle_sync_events(EventKind::State, room, state_events).await?;
+
+        // Event handlers specifically for redacted OR unredacted state events
         self.handle_sync_events_wrapped_with(
             room,
             state_events,
@@ -252,10 +302,12 @@ impl Client {
             |raw| {
                 let StateEventDetails { event_type, unsigned } = raw.deserialize_as()?;
                 let redacted = unsigned.and_then(|u| u.redacted_because).is_some();
-                Ok((EventKind::State { redacted }, event_type))
+                Ok((EventKind::state_redacted(redacted), event_type))
             },
         )
-        .await
+        .await?;
+
+        Ok(())
     }
 
     pub(crate) async fn handle_sync_timeline_events(
@@ -271,6 +323,25 @@ impl Client {
             unsigned: Option<UnsignedDetails>,
         }
 
+        // Event handlers for possibly-redacted timeline events
+        self.handle_sync_events_wrapped_with(
+            room,
+            timeline_events,
+            |e| (&e.event, e.encryption_info.as_ref()),
+            |raw| {
+                let TimelineEventDetails { event_type, state_key, .. } = raw.deserialize_as()?;
+
+                let kind = match state_key {
+                    Some(_) => EventKind::State,
+                    None => EventKind::MessageLike,
+                };
+
+                Ok((kind, event_type))
+            },
+        )
+        .await?;
+
+        // Event handlers specifically for redacted OR unredacted timeline events
         self.handle_sync_events_wrapped_with(
             room,
             timeline_events,
@@ -281,14 +352,16 @@ impl Client {
 
                 let redacted = unsigned.and_then(|u| u.redacted_because).is_some();
                 let kind = match state_key {
-                    Some(_) => EventKind::State { redacted },
-                    None => EventKind::Message { redacted },
+                    Some(_) => EventKind::state_redacted(redacted),
+                    None => EventKind::message_like_redacted(redacted),
                 };
 
                 Ok((kind, event_type))
             },
         )
-        .await
+        .await?;
+
+        Ok(())
     }
 
     async fn handle_sync_events_wrapped_with<'a, T: 'a, U: 'a>(
@@ -303,16 +376,28 @@ impl Client {
             let (ev_kind, ev_type) = get_id(raw_event)?;
             let event_handler_id = (ev_kind, &*ev_type);
 
-            if let Some(handlers) = self.event_handlers.read().await.get(&event_handler_id) {
-                for handler in &*handlers {
+            // Construct event handler futures
+            let futures: Vec<_> = self
+                .event_handlers()
+                .await
+                .get(&event_handler_id)
+                .into_iter()
+                .flatten()
+                .map(|handler| {
                     let data = EventHandlerData {
                         client: self.clone(),
                         room: room.clone(),
                         raw: raw_event.json(),
                         encryption_info,
                     };
-                    matrix_sdk_common::executor::spawn((handler)(data));
-                }
+                    (handler)(data)
+                })
+                .collect();
+
+            // Run the event handler futures with the `self.event_handlers` lock
+            // no longer being held, in order.
+            for fut in futures {
+                fut.await;
             }
         }
 
@@ -354,63 +439,114 @@ mod static_events {
     use ruma::events::{
         self,
         presence::{PresenceEvent, PresenceEventContent},
-        StaticEventContent,
+        EphemeralRoomEventContent, GlobalAccountDataEventContent, MessageLikeEventContent,
+        RedactContent, RedactedEventContent, RoomAccountDataEventContent, StateEventContent,
+        StaticEventContent, ToDeviceEventContent,
     };
 
     use super::{EventKind, SyncEvent};
 
     impl<C> SyncEvent for events::GlobalAccountDataEvent<C>
     where
-        C: StaticEventContent + events::GlobalAccountDataEventContent,
+        C: StaticEventContent + GlobalAccountDataEventContent,
     {
         const ID: (EventKind, &'static str) = (EventKind::GlobalAccountData, C::TYPE);
     }
 
     impl<C> SyncEvent for events::RoomAccountDataEvent<C>
     where
-        C: StaticEventContent + events::RoomAccountDataEventContent,
+        C: StaticEventContent + RoomAccountDataEventContent,
     {
         const ID: (EventKind, &'static str) = (EventKind::RoomAccountData, C::TYPE);
     }
 
     impl<C> SyncEvent for events::SyncEphemeralRoomEvent<C>
     where
-        C: StaticEventContent + events::EphemeralRoomEventContent,
+        C: StaticEventContent + EphemeralRoomEventContent,
     {
         const ID: (EventKind, &'static str) = (EventKind::EphemeralRoomData, C::TYPE);
     }
 
-    impl<C> SyncEvent for events::SyncMessageEvent<C>
+    impl<C> SyncEvent for events::SyncMessageLikeEvent<C>
     where
-        C: StaticEventContent + events::MessageEventContent,
+        C: StaticEventContent + MessageLikeEventContent + RedactContent,
+        C::Redacted: MessageLikeEventContent + RedactedEventContent,
     {
-        const ID: (EventKind, &'static str) = (EventKind::Message { redacted: false }, C::TYPE);
+        const ID: (EventKind, &'static str) = (EventKind::MessageLike, C::TYPE);
+    }
+
+    impl<C> SyncEvent for events::OriginalSyncMessageLikeEvent<C>
+    where
+        C: StaticEventContent + MessageLikeEventContent,
+    {
+        const ID: (EventKind, &'static str) = (EventKind::OriginalMessageLike, C::TYPE);
+    }
+
+    impl<C> SyncEvent for events::RedactedSyncMessageLikeEvent<C>
+    where
+        C: StaticEventContent + MessageLikeEventContent + RedactedEventContent,
+    {
+        const ID: (EventKind, &'static str) = (EventKind::RedactedMessageLike, C::TYPE);
+    }
+
+    impl SyncEvent for events::room::redaction::SyncRoomRedactionEvent {
+        const ID: (EventKind, &'static str) =
+            (EventKind::MessageLike, events::room::redaction::RoomRedactionEventContent::TYPE);
+    }
+
+    impl SyncEvent for events::room::redaction::OriginalSyncRoomRedactionEvent {
+        const ID: (EventKind, &'static str) = (
+            EventKind::OriginalMessageLike,
+            events::room::redaction::RoomRedactionEventContent::TYPE,
+        );
+    }
+
+    impl SyncEvent for events::room::redaction::RedactedSyncRoomRedactionEvent {
+        const ID: (EventKind, &'static str) = (
+            EventKind::RedactedMessageLike,
+            events::room::redaction::RoomRedactionEventContent::TYPE,
+        );
     }
 
     impl<C> SyncEvent for events::SyncStateEvent<C>
     where
-        C: StaticEventContent + events::StateEventContent,
+        C: StaticEventContent + StateEventContent + RedactContent,
+        C::Redacted: StateEventContent + RedactedEventContent,
     {
-        const ID: (EventKind, &'static str) = (EventKind::State { redacted: false }, C::TYPE);
+        const ID: (EventKind, &'static str) = (EventKind::State, C::TYPE);
+    }
+
+    impl<C> SyncEvent for events::OriginalSyncStateEvent<C>
+    where
+        C: StaticEventContent + StateEventContent,
+    {
+        const ID: (EventKind, &'static str) = (EventKind::OriginalState, C::TYPE);
+    }
+
+    impl<C> SyncEvent for events::RedactedSyncStateEvent<C>
+    where
+        C: StaticEventContent + StateEventContent + RedactedEventContent,
+    {
+        const ID: (EventKind, &'static str) = (EventKind::RedactedState, C::TYPE);
     }
 
     impl<C> SyncEvent for events::StrippedStateEvent<C>
     where
-        C: StaticEventContent + events::StateEventContent,
+        C: StaticEventContent + StateEventContent,
     {
         const ID: (EventKind, &'static str) = (EventKind::StrippedState, C::TYPE);
     }
 
     impl<C> SyncEvent for events::InitialStateEvent<C>
     where
-        C: StaticEventContent + events::StateEventContent,
+        C: StaticEventContent + StateEventContent,
     {
         const ID: (EventKind, &'static str) = (EventKind::InitialState, C::TYPE);
     }
 
     impl<C> SyncEvent for events::ToDeviceEvent<C>
     where
-        C: StaticEventContent + events::ToDeviceEventContent,
+        C: StaticEventContent + ToDeviceEventContent,
     {
         const ID: (EventKind, &'static str) = (EventKind::ToDevice, C::TYPE);
     }
@@ -418,40 +554,29 @@ mod static_events {
     impl SyncEvent for PresenceEvent {
         const ID: (EventKind, &'static str) = (EventKind::Presence, PresenceEventContent::TYPE);
     }
-
-    impl<C> SyncEvent for events::RedactedSyncMessageEvent<C>
-    where
-        C: StaticEventContent + events::RedactedMessageEventContent,
-    {
-        const ID: (EventKind, &'static str) = (EventKind::Message { redacted: true }, C::TYPE);
-    }
-
-    impl<C> SyncEvent for events::RedactedSyncStateEvent<C>
-    where
-        C: StaticEventContent + events::RedactedStateEventContent,
-    {
-        const ID: (EventKind, &'static str) = (EventKind::State { redacted: true }, C::TYPE);
-    }
 }
 
-#[cfg(test)]
-mod test {
+#[cfg(all(test, not(target_arch = "wasm32")))]
+mod tests {
+    use matrix_sdk_test::async_test;
+    #[cfg(target_arch = "wasm32")]
+    wasm_bindgen_test::wasm_bindgen_test_configure!(run_in_browser);
     use std::{future, sync::Arc};
 
     use matrix_sdk_test::{EventBuilder, EventsJson};
     use ruma::{
-        events::{room::member::MemberEventContent, StrippedStateEvent, SyncStateEvent},
+        events::room::member::{OriginalSyncRoomMemberEvent, StrippedRoomMemberEvent},
         room_id,
     };
     use serde_json::json;
 
     use crate::{room, Client};
 
-    #[tokio::test]
+    #[async_test]
     async fn event_handler() -> crate::Result<()> {
         use std::sync::atomic::{AtomicU8, Ordering::SeqCst};
 
-        let client = crate::client::test::logged_in_client().await;
+        let client = crate::client::tests::logged_in_client().await;
 
         let member_count = Arc::new(AtomicU8::new(0));
         let typing_count = Arc::new(AtomicU8::new(0));
@@ -461,7 +586,7 @@ mod test {
         client
             .register_event_handler({
                 let member_count = member_count.clone();
-                move |_ev: SyncStateEvent<MemberEventContent>, _room: room::Room| {
+                move |_ev: OriginalSyncRoomMemberEvent, _room: room::Room| {
                     member_count.fetch_add(1, SeqCst);
                     future::ready(())
                 }
@@ -469,7 +594,7 @@ mod test {
             .await
             .register_event_handler({
                 let typing_count = typing_count.clone();
-                move |_ev: SyncStateEvent<MemberEventContent>| {
+                move |_ev: OriginalSyncRoomMemberEvent| {
                     typing_count.fetch_add(1, SeqCst);
                     future::ready(())
                 }
@@ -477,9 +602,7 @@ mod test {
             .await
             .register_event_handler({
                 let power_levels_count = power_levels_count.clone();
-                move |_ev: SyncStateEvent<MemberEventContent>,
-                      _client: Client,
-                      _room: room::Room| {
+                move |_ev: OriginalSyncRoomMemberEvent, _client: Client, _room: room::Room| {
                     power_levels_count.fetch_add(1, SeqCst);
                     future::ready(())
                 }
@@ -487,7 +610,7 @@ mod test {
             .await
             .register_event_handler({
                 let invited_member_count = invited_member_count.clone();
-                move |_ev: StrippedStateEvent<MemberEventContent>| {
+                move |_ev: StrippedRoomMemberEvent| {
                     invited_member_count.fetch_add(1, SeqCst);
                     future::ready(())
                 }
@@ -499,41 +622,41 @@ mod test {
             .add_ephemeral(EventsJson::Typing)
             .add_state_event(EventsJson::PowerLevels)
             .add_custom_invited_event(
-                &room_id!("!test_invited:example.org"),
+                room_id!("!test_invited:example.org"),
                 json!({
-                "content": {
-                  "avatar_url": "mxc://example.org/SEsfnsuifSDFSSEF",
-                  "displayname": "Alice",
-                  "membership": "invite",
-                },
-                "event_id": "$143273582443PhrSn:example.org",
-                "origin_server_ts": 1432735824653u64,
-                "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-                "sender": "@example:example.org",
-                "state_key": "@alice:example.org",
-                "type": "m.room.member",
-                "unsigned": {
-                  "age": 1234,
-                  "invite_room_state": [
-                    {
-                      "content": {
-                        "name": "Example Room"
-                      },
-                      "sender": "@bob:example.org",
-                      "state_key": "",
-                      "type": "m.room.name"
+                    "content": {
+                        "avatar_url": "mxc://example.org/SEsfnsuifSDFSSEF",
+                        "displayname": "Alice",
+                        "membership": "invite",
                     },
-                    {
-                      "content": {
-                        "join_rule": "invite"
-                      },
-                      "sender": "@bob:example.org",
-                      "state_key": "",
-                      "type": "m.room.join_rules"
+                    "event_id": "$143273582443PhrSn:example.org",
+                    "origin_server_ts": 1432735824653u64,
+                    "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
+                    "sender": "@example:example.org",
+                    "state_key": "@alice:example.org",
+                    "type": "m.room.member",
+                    "unsigned": {
+                        "age": 1234,
+                        "invite_room_state": [
+                            {
+                                "content": {
+                                    "name": "Example Room"
+                                },
+                                "sender": "@bob:example.org",
+                                "state_key": "",
+                                "type": "m.room.name"
+                            },
+                            {
+                                "content": {
+                                    "join_rule": "invite"
+                                },
+                                "sender": "@bob:example.org",
+                                "state_key": "",
+                                "type": "m.room.join_rules"
+                            }
+                        ]
                     }
-                  ]
-                }
-                          }),
+                }),
             )
             .build_sync_response();
         client.process_sync(response).await?;
