@@ -12,20 +12,23 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::{convert::TryFrom, fmt::Debug, sync::Arc};
+use std::{any::type_name, convert::TryFrom, fmt::Debug, sync::Arc, time::Duration};
 
+use async_trait::async_trait;
 use bytes::{Bytes, BytesMut};
 use http::Response as HttpResponse;
-use matrix_sdk_common::{async_trait, locks::RwLock, AsyncTraitDeps};
-use reqwest::{Client, Response};
+use matrix_sdk_common::{locks::RwLock, AsyncTraitDeps};
+use reqwest::Response;
 use ruma::api::{
-    client::r0::media::create_content, error::FromHttpResponseError, AuthScheme, IncomingResponse,
-    OutgoingRequest, OutgoingRequestAppserviceExt, SendAccessToken,
+    error::FromHttpResponseError, AuthScheme, IncomingResponse, MatrixVersion, OutgoingRequest,
+    OutgoingRequestAppserviceExt, SendAccessToken,
 };
 use tracing::trace;
 use url::Url;
 
-use crate::{error::HttpError, ClientConfig, RequestConfig, Session};
+use crate::{config::RequestConfig, error::HttpError, Session};
+
+pub(crate) const DEFAULT_REQUEST_TIMEOUT: Duration = Duration::from_secs(10);
 
 /// Abstraction around the http layer. The allows implementors to use different
 /// http libraries.
@@ -49,7 +52,7 @@ pub trait HttpSend: AsyncTraitDeps {
     ///
     /// ```
     /// use std::convert::TryFrom;
-    /// use matrix_sdk::{HttpSend, async_trait, HttpError, RequestConfig, bytes::Bytes};
+    /// use matrix_sdk::{HttpSend, async_trait, HttpError, config::RequestConfig, bytes::Bytes};
     ///
     /// #[derive(Debug)]
     /// struct Client(reqwest::Client);
@@ -106,115 +109,72 @@ impl HttpClient {
         HttpClient { inner, homeserver, session, request_config }
     }
 
-    async fn send_request<Request: OutgoingRequest>(
-        &self,
-        request: Request,
-        session: Arc<RwLock<Option<Session>>>,
-        config: Option<RequestConfig>,
-    ) -> Result<http::Response<Bytes>, HttpError> {
-        let config = match config {
-            Some(config) => config,
-            None => self.request_config,
-        };
-
-        let request = if !self.request_config.assert_identity {
-            self.try_into_http_request(request, session, config).await?
-        } else {
-            self.try_into_http_request_with_identity_assertion(request, session, config).await?
-        };
-
-        self.inner.send_request(request, config).await
-    }
-
-    async fn try_into_http_request<Request: OutgoingRequest>(
-        &self,
-        request: Request,
-        session: Arc<RwLock<Option<Session>>>,
-        config: RequestConfig,
-    ) -> Result<http::Request<Bytes>, HttpError> {
-        let read_guard;
-        let access_token = if config.force_auth {
-            read_guard = session.read().await;
-            if let Some(session) = read_guard.as_ref() {
-                SendAccessToken::Always(session.access_token.as_str())
-            } else {
-                return Err(HttpError::ForcedAuthenticationWithoutAccessToken);
-            }
-        } else {
-            match Request::METADATA.authentication {
-                AuthScheme::AccessToken => {
-                    read_guard = session.read().await;
-
-                    if let Some(session) = read_guard.as_ref() {
-                        SendAccessToken::IfRequired(session.access_token.as_str())
-                    } else {
-                        return Err(HttpError::AuthenticationRequired);
-                    }
-                }
-                AuthScheme::None => SendAccessToken::None,
-                _ => return Err(HttpError::NotClientRequest),
-            }
-        };
-
-        let http_request = request
-            .try_into_http_request::<BytesMut>(
-                &self.homeserver.read().await.to_string(),
-                access_token,
-            )?
-            .map(|body| body.freeze());
-
-        Ok(http_request)
-    }
-
-    async fn try_into_http_request_with_identity_assertion<Request: OutgoingRequest>(
-        &self,
-        request: Request,
-        session: Arc<RwLock<Option<Session>>>,
-        _: RequestConfig,
-    ) -> Result<http::Request<Bytes>, HttpError> {
-        let read_guard = session.read().await;
-        let access_token = if let Some(session) = read_guard.as_ref() {
-            SendAccessToken::Always(session.access_token.as_str())
-        } else {
-            return Err(HttpError::AuthenticationRequired);
-        };
-
-        let user_id = if let Some(session) = read_guard.as_ref() {
-            session.user_id.clone()
-        } else {
-            return Err(HttpError::UserIdRequired);
-        };
-
-        let http_request = request
-            .try_into_http_request_with_user_id::<BytesMut>(
-                &self.homeserver.read().await.to_string(),
-                access_token,
-                user_id,
-            )?
-            .map(|body| body.freeze());
-
-        Ok(http_request)
-    }
-
-    pub async fn upload(
-        &self,
-        request: create_content::Request<'_>,
-        config: Option<RequestConfig>,
-    ) -> Result<create_content::Response, HttpError> {
-        let response = self.send_request(request, self.session.clone(), config).await?;
-        Ok(create_content::Response::try_from_http_response(response)?)
-    }
-
+    #[tracing::instrument(skip(self, request), fields(request_type = type_name::<Request>()))]
     pub async fn send<Request>(
         &self,
         request: Request,
         config: Option<RequestConfig>,
+        server_versions: Arc<[MatrixVersion]>,
     ) -> Result<Request::IncomingResponse, HttpError>
     where
         Request: OutgoingRequest + Debug,
         HttpError: From<FromHttpResponseError<Request::EndpointError>>,
     {
-        let response = self.send_request(request, self.session.clone(), config).await?;
+        let config = match config {
+            Some(config) => config,
+            None => self.request_config,
+        };
+
+        let auth_scheme = Request::METADATA.authentication;
+        if !matches!(auth_scheme, AuthScheme::AccessToken | AuthScheme::None) {
+            return Err(HttpError::NotClientRequest);
+        }
+
+        let access_token;
+
+        let request = if !self.request_config.assert_identity {
+            let send_access_token = if auth_scheme == AuthScheme::None && !config.force_auth {
+                // Small optimization: Don't take the session lock if we know the auth token
+                // isn't going to be used anyways.
+                SendAccessToken::None
+            } else {
+                match self.session.read().await.as_ref() {
+                    Some(session) => {
+                        access_token = session.access_token.clone();
+                        if config.force_auth {
+                            SendAccessToken::Always(&access_token)
+                        } else {
+                            SendAccessToken::IfRequired(&access_token)
+                        }
+                    }
+                    None => SendAccessToken::None,
+                }
+            };
+
+            request.try_into_http_request::<BytesMut>(
+                &self.homeserver.read().await.to_string(),
+                send_access_token,
+                &server_versions,
+            )?
+        } else {
+            let (send_access_token, user_id) = {
+                let session = self.session.read().await;
+                let session = session.as_ref().ok_or(HttpError::UserIdRequired)?;
+
+                access_token = session.access_token.clone();
+                (SendAccessToken::Always(&access_token), session.user_id.clone())
+            };
+
+            request.try_into_http_request_with_user_id::<BytesMut>(
+                &self.homeserver.read().await.to_string(),
+                send_access_token,
+                &user_id,
+                &server_versions,
+            )?
+        };
+
+        let request = request.map(|body| body.freeze());
+        let response = self.inner.send_request(request, config).await?;
 
         trace!("Got response: {:?}", response);
 
@@ -224,43 +184,58 @@ impl HttpClient {
     }
 }
 
-/// Build a client with the specified configuration.
-pub(crate) fn client_with_config(config: &ClientConfig) -> Result<Client, HttpError> {
-    let http_client = reqwest::Client::builder();
-
+#[derive(Debug)]
+pub(crate) struct HttpSettings {
     #[cfg(not(target_arch = "wasm32"))]
-    let http_client = {
-        use http::HeaderValue;
+    pub(crate) disable_ssl_verification: bool,
+    #[cfg(not(target_arch = "wasm32"))]
+    pub(crate) proxy: Option<String>,
+    #[cfg(not(target_arch = "wasm32"))]
+    pub(crate) user_agent: Option<String>,
+    #[cfg(not(target_arch = "wasm32"))]
+    pub(crate) timeout: Duration,
+}
 
-        let http_client = if config.disable_ssl_verification {
-            http_client.danger_accept_invalid_certs(true)
-        } else {
-            http_client
+#[allow(clippy::derivable_impls)]
+impl Default for HttpSettings {
+    fn default() -> Self {
+        Self {
+            #[cfg(not(target_arch = "wasm32"))]
+            disable_ssl_verification: false,
+            #[cfg(not(target_arch = "wasm32"))]
+            proxy: None,
+            #[cfg(not(target_arch = "wasm32"))]
+            user_agent: None,
+            #[cfg(not(target_arch = "wasm32"))]
+            timeout: DEFAULT_REQUEST_TIMEOUT,
+        }
+    }
+}
+
+impl HttpSettings {
+    /// Build a client with the specified configuration.
+    pub(crate) fn make_client(&self) -> Result<reqwest::Client, HttpError> {
+        #[allow(unused_mut)]
+        let mut http_client = reqwest::Client::builder();
+
+        #[cfg(not(target_arch = "wasm32"))]
+        {
+            if self.disable_ssl_verification {
+                http_client = http_client.danger_accept_invalid_certs(true)
+            }
+
+            if let Some(p) = &self.proxy {
+                http_client = http_client.proxy(reqwest::Proxy::all(p.as_str())?);
+            }
+
+            let user_agent =
+                self.user_agent.clone().unwrap_or_else(|| "matrix-rust-sdk".to_owned());
+
+            http_client = http_client.user_agent(user_agent).timeout(self.timeout);
         };
 
-        let http_client = match &config.proxy {
-            Some(p) => http_client.proxy(p.clone()),
-            None => http_client,
-        };
-
-        let mut headers = reqwest::header::HeaderMap::new();
-
-        let user_agent = match &config.user_agent {
-            Some(a) => a.clone(),
-            None => HeaderValue::from_str(&format!("matrix-rust-sdk {}", crate::VERSION))
-                .expect("Can't construct the version header"),
-        };
-
-        headers.insert(reqwest::header::USER_AGENT, user_agent);
-
-        http_client.default_headers(headers).timeout(config.request_config.timeout)
-    };
-
-    #[cfg(target_arch = "wasm32")]
-    #[allow(unused)]
-    let _ = config;
-
-    Ok(http_client.build()?)
+        Ok(http_client.build()?)
+    }
 }
 
 async fn response_to_http_response(
@@ -284,7 +259,7 @@ async fn response_to_http_response(
 
 #[cfg(any(target_arch = "wasm32"))]
 async fn send_request(
-    client: &Client,
+    client: &reqwest::Client,
     request: http::Request<Bytes>,
     _: RequestConfig,
 ) -> Result<http::Response<Bytes>, HttpError> {
@@ -296,7 +271,7 @@ async fn send_request(
 
 #[cfg(all(not(target_arch = "wasm32")))]
 async fn send_request(
-    client: &Client,
+    client: &reqwest::Client,
     request: http::Request<Bytes>,
     config: RequestConfig,
 ) -> Result<http::Response<Bytes>, HttpError> {
@@ -325,7 +300,11 @@ async fn send_request(
         };
 
         // Turn errors into permanent errors when the retry limit is reached
-        let error_type = if stop { RetryError::Permanent } else { RetryError::Transient };
+        let error_type = if stop {
+            RetryError::Permanent
+        } else {
+            |err| RetryError::Transient { err, retry_after: None }
+        };
 
         let request = request.try_clone().ok_or(HttpError::UnableToCloneRequest)?;
 
@@ -355,7 +334,7 @@ async fn send_request(
 
 #[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
 #[cfg_attr(not(target_arch = "wasm32"), async_trait)]
-impl HttpSend for Client {
+impl HttpSend for reqwest::Client {
     async fn send_request(
         &self,
         request: http::Request<Bytes>,
