@@ -1,7 +1,9 @@
 use std::ops::Deref;
 
+use ruma::events::room::MediaSource;
+
 use crate::{
-    media::{MediaFormat, MediaRequest, MediaType},
+    media::{MediaFormat, MediaRequest},
     BaseRoomMember, Client, Result,
 };
 
@@ -46,7 +48,7 @@ impl RoomMember {
     /// # let homeserver = Url::parse("http://example.com").unwrap();
     /// # block_on(async {
     /// # let user = "example";
-    /// let client = Client::new(homeserver).unwrap();
+    /// let client = Client::new(homeserver).await.unwrap();
     /// client.login(user, "password", None, None).await.unwrap();
     /// let room_id = room_id!("!roomid:example.com");
     /// let room = client
@@ -61,7 +63,7 @@ impl RoomMember {
     /// ```
     pub async fn avatar(&self, format: MediaFormat) -> Result<Option<Vec<u8>>> {
         if let Some(url) = self.avatar_url() {
-            let request = MediaRequest { media_type: MediaType::Uri(url.clone()), format };
+            let request = MediaRequest { source: MediaSource::Plain(url.to_owned()), format };
             Ok(Some(self.client.get_media_content(&request, true).await?))
         } else {
             Ok(None)
